typedef unsigned int word;
typedef unsigned char byte;

#define BUFMIN 20
#define BUFMAX QUEUESIZE-10

// nastaveni rizeni toku dat RTS nebo XON/XOFF
#define FLOWRTS 0

#include <8051.h>
#include "mnghw.h"
#include "base.h"
#include "queue.h"

#define XON	0x11
#define XOFF	0x13

#define SetRTS(x) TXD=(x)	// nastavuje bit RTS
bit flow;

TCommandQueue CommandQueue;

byte timeout;	// bit aktivovany nedostatkem znaku - sleep

void ioInit(void)
{
	CommandQueue.Wr = CommandQueue.Rd = 0;

	/* inicializace serioveho portu */
	TMOD = 0x21;	// T0 mode 1, T1 mode 2
	TCON = 0x55;	// run casovace pro RS232, tj T1 a T0
	TH1 = 251;	// 9600 Bd tj. 256 - (18432000 / 12 / 32 / 9600)
	SCON = 0x52;

	ES = 1;
	ET0 = 1;
#if FLOWRTS
	SetRTS(0);
#else
	flow = 1;
	SBUF = XON;
#endif
	timeout = 0;
}

void rsint(void) interrupt 4 using 1
{

//nefiltrovat 0 nebo jine znaky? (RTS muze zlobit)

	char size;

	size = CommandQueue.Rd - CommandQueue.Wr;
	if (size <= 0)
		size += sizeof(CommandQueue.Buf);

	if (size < BUFMIN)	// aspon 16 byte, protoze seriak ma vyrovnavaci pamet
#if FLOWRTS
		SetRTS(1);
#else
		if (flow == 1) {
			SBUF = XOFF;
			flow = 0;
		}
#endif

	if (RI) {
		RI = 0;
		if (CommandQueue.Buf[CommandQueue.Wr] = SBUF)
			if (++CommandQueue.Wr == sizeof(CommandQueue.Buf))
				CommandQueue.Wr = 0;
	}
	TI = 0;
}

//////////////////////////////////////////////////////

byte ReadChar(void)
{
	byte Aux;
	char size;

	// ceka na znak
	while (CommandQueue.Rd == CommandQueue.Wr)	// neni znak ve fronte
		if (timeout > 20) {	// po 1s zvedni
			mngSleep(1);
			timeout = 0;
#if FLOWRTS
			SetRTS(0);
#else
			SBUF = XON;
#endif
		}

	size = CommandQueue.Rd - CommandQueue.Wr;
	if (size <= 0)
		size += sizeof(CommandQueue.Buf);

	// rizeni toku dat
	if (size >= BUFMAX)
#if FLOWRTS
		SetRTS(0);
#else
		if (flow == 0) {
			SBUF = XON;
			flow = 1;
		}
#endif

	Aux = CommandQueue.Buf[CommandQueue.Rd];
	if (++CommandQueue.Rd == sizeof(CommandQueue.Buf))
		CommandQueue.Rd = 0;
	return Aux;
}

void UnReadChar(byte znak)
{
	if (CommandQueue.Rd == 0)
		CommandQueue.Rd = sizeof(CommandQueue.Buf);
	CommandQueue.Buf[--CommandQueue.Rd] = znak;
}

void timeoutfce(void) interrupt 1 using 1
{
	timeout++;
}
